/*
 * Copyright (c) 2018-2019 Atmosphère-NX
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once
#include <switch.h>
#include <stratosphere.hpp>

#include "i2c_driver/i2c_api.hpp"
#include "boot_battery_parameters.hpp"

class BatteryDriver {
    private:
        I2cSessionImpl i2c_session;
    public:
        BatteryDriver() {
            I2cDriver::Initialize();
            I2cDriver::OpenSession(&this->i2c_session, I2cDevice_Max17050);
        }

        ~BatteryDriver() {
            I2cDriver::CloseSession(this->i2c_session);
            I2cDriver::Finalize();
        }
    private:
        static const Max17050Parameters *GetBatteryParameters();

        Result Read(u8 addr, u16 *out_data);
        Result Write(u8 addr, u16 val);
        Result ReadWrite(u8 addr, u16 mask, u16 val);
        bool WriteValidate(u8 addr, u16 val);

        bool IsPowerOnReset();
        Result LockVfSoc();
        Result UnlockVfSoc();
        Result LockModelTable();
        Result UnlockModelTable();
        bool IsModelTableLocked();
        Result SetModelTable(const u16 *model_table);
        bool IsModelTableSet(const u16 *model_table);

    public:
        Result InitializeBatteryParameters();
        Result IsBatteryRemoved(bool *out);
        Result GetTemperature(double *out);
        Result GetAverageVCell(u32 *out);
        Result GetSocRep(double *out);
        Result GetBatteryPercentage(size_t *out);
        Result SetShutdownTimer();
        Result GetShutdownEnabled(bool *out);
        Result SetShutdownEnabled(bool enabled);
};
